#include "ImageWidget.h"
#include "TargaImage.h"
#include <Fl/Fl_Window.h>
#include <Fl/Fl_Button.h>
#include <Fl/Fl_File_Chooser.h>
#include <Fl/fl_draw.h>
#include <libtarga.h>
#include <string.h>


// constants
const int   c_border                = 10;                                       // border width between window elements in pixels
const int   c_buttonHeight          = 30;                                       // button height in pixels
const int   c_buttonWidth           = 100;                                      // button width in pixels
const int   c_buttonPaneHeight      = 2 * c_border + c_buttonHeight;            // height of pane for buttons in pixels
const int   c_minWindowWidth        = 6 * c_border + 3 * c_buttonWidth;         // minimum window width in pixels



///////////////////////////////////////////////////////////////////////////////
//
//      Get the maximum of two values.  Note:  operator < must be defined for
//  the given type.
//
///////////////////////////////////////////////////////////////////////////////
template<class Type> inline Type Max(Type valueA, Type valueB)
{
    return ((valueA < valueB) ? valueB : valueA);
}// Max


///////////////////////////////////////////////////////////////////////////////
//
//      Constructor.  Add the buttons to the window.
//
///////////////////////////////////////////////////////////////////////////////
ImageWidget::ImageWidget(int x, int y, int w, int h, char *title) : Fl_Widget(x, y, Max(w, c_minWindowWidth), h, title), m_pImage(NULL)
{
    m_pImage = NULL;

    
    // add buttons
    int horizontalCenter = Max(w, c_minWindowWidth) / 2;
    int verticalButtonPos = h - c_border - c_buttonHeight;
    
    m_pLoadButton = new Fl_Button(horizontalCenter - static_cast<int>(c_buttonWidth * 1.5f) - 2 * c_border, verticalButtonPos, c_buttonWidth, c_buttonHeight, "Load Image");
    m_pLoadButton->callback(LoadButtonCallback, this);

    m_pSaveButton = new Fl_Button(horizontalCenter - c_buttonWidth / 2, verticalButtonPos, c_buttonWidth, c_buttonHeight, "Save Image");
    m_pSaveButton->callback(SaveButtonCallback, this);

    m_pModifyButton = new Fl_Button(horizontalCenter + c_buttonWidth / 2 + 2 * c_border, verticalButtonPos, c_buttonWidth, c_buttonHeight, "Modify Image");
    m_pModifyButton->callback(ModifyButtonCallback, this);
}// ImageWidget


///////////////////////////////////////////////////////////////////////////////
//
//      Set the image to be displayed, position window elements, and show the
//  image.
//
///////////////////////////////////////////////////////////////////////////////
void ImageWidget::Set_Image(TargaImage *i)
{
    Fl_Widget* pParent = parent();

    m_pImage = i;

    // Resize the parent, which resizes this window, and force a redraw.
    pParent->size(Max(m_pImage->width, c_minWindowWidth), m_pImage->height + c_buttonPaneHeight);
    PositionButtons(m_pImage->width, m_pImage->height);  

    pParent->redraw();
}// Set_Image


///////////////////////////////////////////////////////////////////////////////
//
//      Get the current image.
//
///////////////////////////////////////////////////////////////////////////////
TargaImage* ImageWidget::Get_Image(void)
{
    return m_pImage;
}// Get_Image


///////////////////////////////////////////////////////////////////////////////
//
//      Draw the window contents.
//
///////////////////////////////////////////////////////////////////////////////
void ImageWidget::draw(void)
{
    unsigned char   *rgb;

    if (!m_pImage)          // Don't do anything if the image is empty.
    	return;

    rgb = m_pImage->To_RGB(); // Convert the pre-multiplied RGBA image into RGB.
    fl_draw_image(rgb, x(), y(), m_pImage->width, m_pImage->height, 3);
    delete[] rgb;
}// draw


///////////////////////////////////////////////////////////////////////////////
//
//      Draw the image in the window.
//
///////////////////////////////////////////////////////////////////////////////
void ImageWidget::Redraw_Image(void)
{
    redraw();
}// Redraw_Image


///////////////////////////////////////////////////////////////////////////////
//
//      Position the buttons given the image dimensions.
//
///////////////////////////////////////////////////////////////////////////////
void ImageWidget::PositionButtons(int imageWidth, int imageHeight)
{
    int horizontalCenter = Max(imageWidth, c_minWindowWidth) / 2;

    m_pLoadButton->position(horizontalCenter - static_cast<int>(c_buttonWidth * 1.5f) - 2 * c_border, imageHeight + c_border);
    m_pSaveButton->position(horizontalCenter - c_buttonWidth / 2, imageHeight + c_border);
    m_pModifyButton->position(horizontalCenter + c_buttonWidth / 2 + 2 * c_border, imageHeight + c_border);
}// void


///////////////////////////////////////////////////////////////////////////////
//
//      Callback for load image button.
//
///////////////////////////////////////////////////////////////////////////////
void ImageWidget::LoadButtonCallback(Fl_Widget* pWidget, void* pData)
{
    char* sFilename = fl_file_chooser("Open Image", NULL, NULL);
    if (sFilename && strlen(sFilename))
    {
        TargaImage* pImage = TargaImage::Load_Image(sFilename);
        
        if (pImage)
        {
            ImageWidget* pImageWidget = static_cast<ImageWidget*>(pData);
            
            if (pImageWidget->m_pImage)
                delete pImageWidget->m_pImage;

            pImageWidget->Set_Image(pImage);
        }// if
    }// if
}// LoadButtonCallback


///////////////////////////////////////////////////////////////////////////////
//
//      Callback for save image button.
//
///////////////////////////////////////////////////////////////////////////////
void ImageWidget::SaveButtonCallback(Fl_Widget* pWidget, void* pData)
{
    if (static_cast<ImageWidget*>(pData)->m_pImage)
    {
        char* sFilename = fl_file_chooser("Save Image", NULL, NULL);
        
        if (sFilename && strlen(sFilename))
            static_cast<ImageWidget*>(pData)->m_pImage->Save_Image(sFilename);
    }// if
}// SaveButtonCallback


///////////////////////////////////////////////////////////////////////////////
//
//      Callback for modify image button.
//
///////////////////////////////////////////////////////////////////////////////
void ImageWidget::ModifyButtonCallback(Fl_Widget* pWidget, void* pData)
{
    ImageWidget* pImageWidget = static_cast<ImageWidget*>(pData);
    if (pImageWidget->m_pImage)
    {
        pImageWidget->m_pImage->Modify();
        pImageWidget->Set_Image(pImageWidget->m_pImage);
    }// if
}// ModifyButtonCallback
